unit yokeipc;
{$mode objfpc}{$H+}
interface
uses
  Classes, SysUtils, simpleipc;
(* Lazarus' IPC server and client are useful for sending messages between programs.
Traditionally many clients can send string messages to a single server.
However, we have two unusual requirements:
 1.) we want multiple programs to send float data to multiple servers
 2.) we want to send float data.
      Converting floats to string format and then back is slow and risks precision loss
      Therefore we pack 3 single precision floats into a 12-character 'string'
 3.) we do not want to generate overhead when a message is repeated, so only novel messages
      are sent and GetIPCFloats will only return true for a novel message
      To disable this feature, remove lines referring to 'gLastMsg'
*)
procedure startipcserver (var S: TSimpleIPCServer; var C: TSimpleIPCClient);
procedure SendIPCFloats(var C: TSimpleIPCClient; s1,s2,s3: single);
function GetIPCFloats(Str: string; var s1,s2,s3: single): boolean;

implementation

const
kIPCServerBase = 'mricro';
knIPCServer=10;
var
   gLastMsg: string;

//IPCserver only passes strings, converting floats to and from strings would be slow and risk loss in precision
//Here we pack our floats into what appears to be a string...
function single2string( s1,s2,s3: single): string;
type
  swaptype = packed record
    case byte of
      0:(c0,c1,c2,c3 : char);
      1:(s:single);
  end;
var
  v:swaptype;
begin
  v.s := s1;
  result := v.c0+v.c1+v.c2+v.c3;
  v.s := s2;
  result := result+v.c0+v.c1+v.c2+v.c3;
  v.s := s3;
  result := result+v.c0+v.c1+v.c2+v.c3;
end;//single2string

procedure string2single(str: string; var s1,s2,s3: single);
type
  swaptype = packed record
    case byte of
      0:(c0,c1,c2,c3 : char);
      1:(s:single);
  end;
var
  v:swaptype;
begin
  v.c0 := str[1];
  v.c1 := str[2];
  v.c2 := str[3];
  v.c3 := str[4];
  s1 := v.s;
  v.c0 := str[5];
  v.c1 := str[6];
  v.c2 := str[7];
  v.c3 := str[8];
  s2 := v.s;
  v.c0 := str[9];
  v.c1 := str[10];
  v.c2 := str[11];
  v.c3 := str[12];
  s3 := v.s;
end;//single2string

procedure startipcserver (var S: TSimpleIPCServer; var C: TSimpleIPCClient);
var
  id: string;
  i: integer;
begin
  s.Active:=false;
  s.Global := true;
  i := 0;
  repeat
        inc(i);
        s.ServerID := kIPCServerBase+inttostr(i);
        c.ServerID := kIPCServerBase+inttostr(i);
  until (i = knIPCServer) or (not c.ServerRunning);
  s.Active:=True;
end;

procedure SendIPCFloats(var C: TSimpleIPCClient; s1,s2,s3: single);
var
 i : integer;
 str : string;
begin
  str := single2string(s1,s2,s3);
  if Str = gLastMsg then
     exit;
  for i := 1 to knIPCServer do begin
      C.Active:=False;
      C.ServerID := kIPCServerBase+inttostr(i);
      if C.ServerRunning then begin
         C.Active:=True;
         C.SendStringMessage(str);
         C.Active:=False;
       end;
  end;
end;

function GetIPCFloats(Str: string; var s1,s2,s3: single): boolean;
begin
     result := false;
     if Str = gLastMsg then
        exit;
     gLastMsg := Str;
     string2single(Str,s1,s2,s3);
     result := true;
end;

end.

